@@ -1,5 +1,6 @@
 from . import primitives
 from . import transforms
+from . import vector_text
 
 import datetime
 import math
@@ -8,10 +9,13 @@ class Plot:
     def __init__(
         self,
         title='plot',
+        *,
         transform=None,
         hide_axes=False,
         primitive=None,
         datetime_unit=1,
+        legend_displacement=(0, -1),
+        legend_offset=(0, -1),
     ):
         self.title = title
         self.points = []
@@ -19,6 +23,7 @@ class Plot:
         self.rects = []
         self.late_vertexors = []
         self.texts = []
+        self.static_texter = vector_text.Texter()
         self.x_min =  math.inf
         self.x_max = -math.inf
         self.y_min =  math.inf
@@ -29,28 +34,24 @@ class Plot:
         self.hide_axes = hide_axes
         self.set_primitive(primitive or primitives.Point())
         self.datetime_unit = datetime_unit
+        self.legend_displacement = legend_displacement
+        self.legend_offset = legend_offset
 
     def point(self, x, y, r=255, g=255, b=255, a=255):
-        x, y = self._to_screen(x, y)
         self.points.append([x, y, r, g, b, a])
         self._include(x, y)
 
     def line(self, xi, yi, xf, yf, r=255, g=255, b=255, a=255):
-        xi, yi = self._to_screen(xi, yi)
-        xf, yf = self._to_screen(xf, yf)
         self.lines.append([xi, yi, xf, yf, r, g, b, a])
         self._include(xi, yi)
         self._include(xf, yf)
 
     def rect(self, xi, yi, xf, yf, r=255, g=255, b=255, a=255):
-        xi, yi = self._to_screen(xi, yi)
-        xf, yf = self._to_screen(xf, yf)
         self.rects.append([xi, yi, xf, yf, r, g, b, a])
         self._include(xi, yi)
         self._include(xf, yf)
 
     def late_vertexor(self, vertexor, x, y, r=255, g=255, b=255, a=255):
-        x, y = self._to_screen(x, y)
         self.late_vertexors.append([vertexor, x, y, r, g, b, a])
         self._include(x, y)
 
@@ -59,6 +60,13 @@ class Plot:
         self.texts.append([s, x, y, r, g, b, a, max_w, max_h, scale])
         self._include(x, y)
 
+    def text_static(self, s, x, y, w=1, h=1, r=255, g=255, b=255, a=255):
+        '`w` and `h` are for a single character.'
+        if not s: return
+        self.static_texter.text(s, x, y, w, h, r, g, b, a)
+        self._include(*self.static_texter.bounds[0:2])
+        self._include(*self.static_texter.bounds[2:4])
+
     def show(self, w=640, h=480):
         from .show import show
         show(self, w, h)
@@ -104,34 +112,61 @@ class Plot:
         self._plot_common(**kwargs)
 
     def plot_2d(self, array, **kwargs):
-        v_min = math.inf
-        v_max = -math.inf
+        z_min = math.inf
+        z_max = -math.inf
         for row in array:
-            v_min = min(v_min, min(row))
-            v_max = max(v_max, max(row))
-        v_rng = v_max - v_min
-        if v_rng == 0: return
+            z_min = min(z_min, min(row))
+            z_max = max(z_max, max(row))
+        z_rng = z_max - z_min
+        if z_rng == 0: return
         i = 0
         for y, row in enumerate(array):
-            for x, v in enumerate(row):
-                kwargs = {'a': float((v - v_min) / v_rng)}
+            for x, z in enumerate(row):
+                rect = {'a': float((z - z_min) / z_rng)}
                 a = self.transform(x, y, i, self.series)
-                kwargs['xi'] = a['x']
-                kwargs['yi'] = a['y']
-                if 'r' in a: kwargs['r'] = a['r']
-                if 'g' in a: kwargs['g'] = a['g']
-                if 'b' in a: kwargs['b'] = a['b']
+                rect['xi'] = a['x']
+                rect['yi'] = a['y']
+                if 'r' in a: rect['r'] = a['r']
+                if 'g' in a: rect['g'] = a['g']
+                if 'b' in a: rect['b'] = a['b']
                 b = self.transform(x+1, y+1, i, self.series)
-                kwargs['xf'] = b['x']
-                kwargs['yf'] = b['y']
-                self.rect(**kwargs)
+                rect['xf'] = b['x']
+                rect['yf'] = b['y']
+                self.rect(**rect)
+        self._plot_common(**kwargs)
+
+    def plot_heatmap(self, triples, **kwargs):
+        z_min = math.inf
+        z_max = -math.inf
+        zs = [i[2] for i in triples]
+        z_min = min(zs)
+        z_max = max(zs)
+        z_rng = z_max - z_min
+        if z_rng == 0: return
+        i = 0
+        for x, y, z in triples:
+            rect = {'a': float((z - z_min) / z_rng)}
+            a = self.transform(x, y, i, self.series)
+            rect['xi'] = a['x']
+            rect['yi'] = a['y']
+            if 'r' in a: rect['r'] = a['r']
+            if 'g' in a: rect['g'] = a['g']
+            if 'b' in a: rect['b'] = a['b']
+            b = self.transform(x+1, y+1, i, self.series)
+            rect['xf'] = b['x']
+            rect['yf'] = b['y']
+            self.rect(**rect)
+        self._plot_common(**kwargs)
+
+    def plot_empty(self, *args, **kwargs):
         self._plot_common(**kwargs)
 
     def plot(self, *args, **kwargs):
         plot_func = None
-        if len(args) == 1:
-            if args[0] == []: return self
-            elif _is_dim(args[0], 1): plot_func = self.plot_list
+        if len(args) >= 1 and _is_empty(args[0]):
+            plot_func = self.plot_empty
+        elif len(args) == 1:
+            if   _is_dim(args[0], 1): plot_func = self.plot_list
             elif _is_dim(args[0], 2): plot_func = self.plot_lists
             elif _type_r(args[0], 1) == _type_r([()]): plot_func = self.plot_scatter_pairs
             elif type(args[0]) == dict: plot_func = self.plot_dict
@@ -155,34 +190,40 @@ class Plot:
         self.primitive = primitive.set_plot(self)
 
     def _include(self, x, y):
-        self.x_min = min(x, self.x_min)
-        self.x_max = max(x, self.x_max)
-        self.y_min = min(y, self.y_min)
-        self.y_max = max(y, self.y_max)
-
-    def _to_screen(self, x, y):
-        c = [x, y]
-        for i, v in enumerate(c):
-            if type(v) == datetime.datetime:
-                if i not in self.epochs:
-                    self.epochs[i] = v
-                c[i] = (v - self.epochs[i]).total_seconds() / self.datetime_unit
-        return c[0], c[1]
+        if type(x) == datetime.datetime:
+            if 'x' in self.epochs:
+                self.epochs['x']['min'] = min(x, self.epochs['x']['min'])
+                self.epochs['x']['max'] = max(x, self.epochs['x']['max'])
+            else:
+                self.epochs['x'] = {'min': x, 'max': x}
+        else:
+            self.x_min = min(x, self.x_min)
+            self.x_max = max(x, self.x_max)
+        if type(y) == datetime.datetime:
+            if 'y' in self.epochs:
+                self.epochs['y']['min'] = min(y, self.epochs['y']['min'])
+                self.epochs['y']['max'] = max(y, self.epochs['y']['max'])
+            else:
+                self.epochs['y'] = {'min': y, 'max': y}
+        else:
+            self.y_min = min(y, self.y_min)
+            self.y_max = max(y, self.y_max)
 
     def _plot_common(
         self,
         next_series=True,
         legend=None,
-        legend_displacement=(0, -1),
-        legend_offset=(0, -1),
         **kwargs,
     ):
         if legend:
-            h = (self.y_max - self.y_min) / 24
             kwargs = self.transform(0, 0, 0, self.series)
-            kwargs['x'] += legend_displacement[0] * self.series + legend_offset[0]
-            kwargs['y'] += legend_displacement[1] * self.series + legend_offset[1]
-            self.text(legend, max_h=h, **kwargs)
+            if hasattr(self.transform, 'series_insignificant'):
+                series = self.transform.series_insignificant(self.series)
+            else:
+                series = self.series
+            kwargs['x'] += self.legend_displacement[0] * series + self.legend_offset[0]
+            kwargs['y'] += self.legend_displacement[1] * series + self.legend_offset[1]
+            self.text(legend, max_h=abs(self.legend_displacement[1]) or 1, **kwargs)
         if next_series:
             self.next_series()
 
@@ -196,9 +237,9 @@ def plot(
 ):
     Plot(
         title,
-        transform,
-        hide_axes,
-        primitive,
+        transform=transform,
+        hide_axes=hide_axes,
+        primitive=primitive,
     ).plot(*args, **kwargs).show()
 
 def _type_r(v, max_depth=None, _depth=0):
@@ -215,3 +256,9 @@ def _is_dim(v, dim):
     u = 0
     for i in range(dim): u = [u]
     return _type_r(v, dim) == _type_r(u)
+
+def _is_empty(v):
+    try:
+        return len(v) == 0
+    except:
+        return False
