@@ -1,6 +1,7 @@
 from . import media
 
 import copy
+import datetime
 import math
 
 class View:
@@ -18,6 +19,39 @@ def fcolor(r, g, b, a):
         for i in [r, g, b, a]
     ]
 
+def translate_x_date(plot, list_, component):
+    for i in list_:
+        x = i[component]
+        x = (x - plot.epochs['x']['min']).total_seconds() / plot.datetime_unit
+        i[component] = x
+
+def translate_y_date(plot, list_, component):
+    for i in list_:
+        y = i[component]
+        y = (y - plot.epochs['y']['min']).total_seconds() / plot.datetime_unit
+        i[component] = y
+
+def translate_dates(plot):
+    if not plot.epochs: return
+    if 'x' in plot.epochs:
+        translate_x_date(plot, plot.late_vertexors, 1)
+        translate_x_date(plot, plot.points, 0)
+        translate_x_date(plot, plot.lines, 0)
+        translate_x_date(plot, plot.lines, 2)
+        translate_x_date(plot, plot.rects, 0)
+        translate_x_date(plot, plot.rects, 2)
+        plot.x_max = max(plot.x_max, (plot.epochs['x']['max'] - plot.epochs['x']['min']).total_seconds() / plot.datetime_unit)
+        plot.x_min = min(plot.x_min, 0)
+    if 'y' in plot.epochs:
+        translate_y_date(plot, plot.late_vertexors, 2)
+        translate_y_date(plot, plot.points, 1)
+        translate_y_date(plot, plot.lines, 1)
+        translate_y_date(plot, plot.lines, 3)
+        translate_y_date(plot, plot.rects, 1)
+        translate_y_date(plot, plot.rects, 3)
+        plot.y_max = max(plot.y_max, (plot.epochs['y']['max'] - plot.epochs['y']['min']).total_seconds() / plot.datetime_unit)
+        plot.y_min = min(plot.y_min, 0)
+
 def construct(plot, view, w, h):
     plot.buffer = media.Buffer()
     plot.buffer_text = media.Buffer()
@@ -35,10 +69,13 @@ def construct(plot, view, w, h):
     for x, y, r, g, b, a in plot.points:
         plot.buffer.add(x, y, *fcolor(r, g, b, a))
     points_f = len(plot.buffer)
-    # lines
+    # lines & static text
     for xi, yi, xf, yf, r, g, b, a in plot.lines:
         plot.buffer.add(xi, yi, *fcolor(r, g, b, a))
         plot.buffer.add(xf, yf, *fcolor(r, g, b, a))
+    for i in range(0, len(plot.static_texter.data), 6):
+        x, y, r, g, b, a = plot.static_texter.data[i:i+6]
+        plot.buffer.add(x, y, *fcolor(r, g, b, a))
     lines_f = len(plot.buffer)
     # rects
     for xi, yi, xf, yf, r, g, b, a in plot.rects:
@@ -59,6 +96,7 @@ def construct(plot, view, w, h):
 
 def show(plot, w, h):
     media.init(w, h, title=plot.title)
+    translate_dates(plot)
     if plot.x_min == plot.x_max:
         plot.x_min -= 1
         plot.x_max += 1
@@ -164,7 +202,7 @@ def show(plot, w, h):
             text_w = 10 / media.width()  * view.w
             text_h = 15 / media.height() * view.h
             # draw x axis
-            increment = 10 ** int(math.log10(view.w))
+            increment = 10 ** math.floor(math.log10(view.w))
             if view.w / increment < 2:
                 increment /= 5
             elif view.w / increment < 5:
@@ -173,20 +211,20 @@ def show(plot, w, h):
             while i < view.x + view.w:
                 s = '{:.5}'.format(i)
                 if view.x + view.w - i > increment:
-                    if i == 0 and plot.epochs.get(0) != None:
+                    if i == 0 and plot.epochs.get('x') != None:
                         texter.text(
-                            plot.epochs[0].isoformat('\n'),
+                            plot.epochs['x']['min'].isoformat('\n'),
                             x=i + margin_x,
-                            y=view.y + margin_y + text_h,
-                            w=text_w / 2,
-                            h=text_h / 2,
+                            y=view.y + margin_y + 1.33 * text_h,
+                            w=text_w * 0.66,
+                            h=text_h * 0.66,
                         )
                     else:
                         texter.text(s, x=i+margin_x, y=view.y+margin_y, w=text_w, h=text_h)
                     texter.text('L', i, view.y, text_w * 2, text_h)
                 i += increment
             # draw y axis
-            increment = 10 ** int(math.log10(view.h))
+            increment = 10 ** math.floor(math.log10(view.h))
             if view.h / increment < 2:
                 increment /= 5
             elif view.h / increment < 5:
@@ -194,13 +232,13 @@ def show(plot, w, h):
             i = (view.y + text_h + 2*margin_y) // increment * increment + increment
             while i < view.y + view.h - (text_h + 2*margin_y):
                 s = '{:.5}'.format(i)
-                if i == 0 and plot.epochs.get(1) != None:
+                if i == 0 and plot.epochs.get('y') != None:
                     texter.text(
-                        plot.epochs[1].isoformat('\n'),
+                        plot.epochs['y']['min'].isoformat('\n'),
                         x=view.x + margin_x,
-                        y=i + margin_y + text_h,
-                        w=text_w / 2,
-                        h=text_h / 2,
+                        y=i + margin_y + 1.33 * text_h,
+                        w=text_w * 0.66,
+                        h=text_h * 0.66,
                     )
                 else:
                     texter.text(s, x=view.x+margin_x, y=i+margin_y, w=text_w, h=text_h)
